@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.c,v 1.67 2005/04/13 17:53:28 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/functions.c,v 1.72 2005/09/15 16:45:27 richb Exp $
  *
  *  Copyright (c) 1987-2005 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -416,8 +416,19 @@ do_expression()
 	        v->e.calc_complete = 1;
 		goto out;
             } else {
-	        update_statusbar(_("Malformed expression"), 
-                                 "gtk-dialog-error");
+		char *message = NULL;
+		switch (ret) {
+		case -PARSER_ERR_INVALID_BASE:
+		    message = _("Invalid number for the current base");
+		    break;
+		case -PARSER_ERR_TOO_LONG_NUMBER:
+		    message = _("Too long number");
+		    break;
+		default:
+		    message = _("Malformed expression");
+		    break;
+		}
+	        update_statusbar(message, "gtk-dialog-error");
 	        return;
             }
         } else {
@@ -443,7 +454,7 @@ void
 do_calc()      /* Perform arithmetic calculation and display result. */
 {
     double dval, dres;
-    int MP1[MP_SIZE];
+    int MP1[MP_SIZE], MP2[MP_SIZE];
 
     if (!(v->opsptr && !v->show_paren)) {  /* Don't do if processing parens. */
         if (key_equal(v->current, KEY_EQ) && 
@@ -480,6 +491,18 @@ do_calc()      /* Perform arithmetic calculation and display result. */
     } else if (IS_KEY(v->cur_op, KEY_DIV.value[0])) {     /* Division. */
         mpdiv(v->MPresult, v->MPdisp_val, v->MPresult);
 
+    } else if (IS_KEY(v->cur_op, KEY_MOD.value[0])) {     /* Modulus. */
+        mpcmim(v->MPresult, MP1);
+        mpcmim(v->MPdisp_val, MP2);
+        if (!mpeq(v->MPresult, MP1) || !mpeq(v->MPdisp_val, MP2)) {
+            doerr(_("Error, operands must be integers"));
+        }
+
+        mpdiv(v->MPresult, v->MPdisp_val, MP1);
+        mpcmim(MP1, MP1);
+        mpmul(MP1, v->MPdisp_val, MP2);
+        mpsub(v->MPresult, MP2, v->MPresult);
+
     } else if (IS_KEY(v->cur_op, KEY_PER.value[0])) {     /* % */
         mpmul(v->MPresult, v->MPdisp_val, v->MPresult);
         MPstr_to_num("0.01", DEC, MP1);
@@ -571,8 +594,6 @@ do_tfunc(int s[MP_SIZE], int t[MP_SIZE], enum trig_func tfunc)
 {
     // Assumes the SIN=0, COS=1, TAN=2
     
-    assert(tfunc < 3); 
-    
     enum trig_func conv_table[3][4] = {
 	{sin_t, asin_t, sinh_t, asinh_t},
 	{cos_t, acos_t, cosh_t, acosh_t},
@@ -582,6 +603,8 @@ do_tfunc(int s[MP_SIZE], int t[MP_SIZE], enum trig_func tfunc)
     int inverse = (v->inverse) ? 1 : 0;
     int hyperbolic = (v->hyperbolic) ? 2 : 0;
     int mode = (inverse | hyperbolic);
+
+    assert(tfunc < 3); 
     
     if (!v->current) return -EINVAL;
     
@@ -894,7 +917,18 @@ do_immedfunc(int s[MP_SIZE], int t[MP_SIZE])
         mpstr(s, MP1);
         mpmul(MP1, MP1, t);
     } else if (key_equal(v->current, KEY_CHS)) {          /* +/- */
-        mpneg(s, t);
+         if (v->key_exp) {
+             if (*v->exp_posn == '+') {
+                 *v->exp_posn = '-';
+             } else {
+                 *v->exp_posn = '+';
+             }
+             set_display(v->display, FALSE);
+             MPstr_to_num(v->display, v->base, s);
+             v->key_exp = 0;
+         } else {
+             mpneg(s, t);
+        }
     }
 }
 
@@ -979,11 +1013,36 @@ do_numtype(enum num_type n)   /* Set number display type. */
 {
     v->dtype = n;
     put_resource(R_DISPLAY, Rdstr[(int) v->dtype]);
-    v->pending = 0;
-    show_display(v->MPdisp_val);
-    if (v->rstate) {
-        make_registers();
+
+    switch (v->syntax) {
+    case npa:
+        v->pending = 0;
+        if (v->rstate) {
+            make_registers();
+        }
+        break;
+        
+    case exprs: {
+        int MP[MP_SIZE];
+        int ret = usable_num(MP);
+        if (ret) {
+            update_statusbar(_("No sane value to convert"),
+                             "gtk-dialog-error");
+        } else {
+            mpstr(v->e.ans, v->e.ansbak);
+            mpstr(MP, v->e.ans);
+            exp_replace("Ans");
+            v->e.calc_complete = 1;
+            make_registers();
+        }
     }
+    break;
+
+    default:
+        assert(0);
+    }	
+
+    refresh_display();
 }
 
 
