@@ -1,5 +1,5 @@
 
-/*  $Header: /cvs/gnome/gcalctool/gcalctool/graphics.c,v 1.28 2005/02/07 16:38:58 richb Exp $
+/*  $Header: /cvs/gnome/gcalctool/gcalctool/graphics.c,v 1.31 2005/09/13 21:38:55 richb Exp $
  *
  *  Copyright (c) 1987-2005 Sun Microsystems, Inc. All Rights Reserved.
  *           
@@ -51,26 +51,12 @@ void
 make_registers()            /* Calculate memory register frame values. */
 {
     char line[MAXLINE];     /* Current memory register line. */
-    char fmt[MAXLINE];      /* Format string to create the register string. */
     char *mval;
     int n;
 
     for (n = 0; n < MAXREGS; n++) {
-        mval = make_number(v->MPmvals[n], v->base, FALSE, TRUE);
-	SPRINTF(fmt, "<span weight=\"bold\">%s%s%%%1ds", 
-                _("R"), "%1d:</span>   %s", MAX_DIGITS - strlen(mval));
-       SPRINTF(line, fmt, n, mval, " ");
-        make_reg(n, line);
+        mval = make_number(v->MPmvals[n], v->base, TRUE);
+        make_reg(n, mval);
+        put_resource(((int) R_REG0) + n, mval);
     }
 }
-
-
-/* XXX: is this routine needed anymore? */
-
-#if 0
-void
-process_str(char *str)
-{
-
-}
-#endif
